import { env, randomUUIDv7 } from 'bun';
import homepage from '@routes/index.html';
import robotsTxt from '@static/robots.txt';
import sitemapTxt from '@static/sitemap.txt';
import favicon from '@static/favicon.png';
import ptApi from '@server/pt-api';
import auth from '@server/auth';

const development = env.NODE_ENV !== 'production';

const faviconInit = { headers: new Headers({ 'Content-Type': 'image/png' }) };

let entriesCache = '';
let entriesCacheTime = 0;
const entriesCacheTimeLimit = 1000 * 15;

async function getEntriesCached() {
  if (entriesCacheTime + entriesCacheTimeLimit < Date.now()) {
    const data = await ptApi.query();
    entriesCache = JSON.stringify(data);
    entriesCacheTime = Date.now();
  }
  return entriesCache;
}

Bun.serve({
  routes: {
    '/': homepage,
    '/robots.txt': new Response(robotsTxt, {
      headers: { 'Content-Type': 'text/plain' },
    }),
    '/sitemap.txt': new Response(sitemapTxt, {
      headers: { 'Content-Type': 'text/plain' },
    }),
    '/favicon.ico': new Response(favicon, faviconInit),
    '/health': new Response('OK'),
    '/api/entries': {
      async GET(req) {
        const session = await auth.verify(req.headers);
        if (!session) {
          return auth.verifyFailResponse();
        }

        return new Response(await getEntriesCached(), {
          headers: {
            'Content-Type': 'application/json'
          }
        });
      },
    },

    '/auth/logout': {
      async POST() {
        return auth.logoutResponse();
      },
    },
    '/auth/verify': {
      async GET(req) {
        const session = await auth.verify(req.headers);
        if (!session) {
          return auth.verifyFailResponse();
        }

        return auth.verifyResponse(session.token);
      },
    },
    '/auth/login': {
      async POST(req) {
        const json = await req.json();
        const data = json as { email?: string, password?: string };
        const email = data.email?.toLocaleLowerCase();
        const password = data.password;

        if (
          typeof email !== 'string' ||
          typeof password !== 'string' ||
          email.length < 3 ||
          password.length === 0
        ) {
          return new Response('Missing email or password', { status: 400 });
        }

        const token = await auth.login(email, password);

        if (!token) {
          await new Promise((resolve) => setTimeout(resolve, (Math.random() * 200) + 800));
          return new Response('Incorrect email or password', {
            status: 400,
          });
        }

        return auth.loginResponse(token)
      },
    },
  },
  development,
  reusePort: true,
  // async fetch(req, server) {
  //   return new Response("Not found", { status: 404 });
  // },
});
