import { eq } from "drizzle-orm";
import { drizzleDB } from "./db";
import { users, userSessions } from "./db/schema";
import { env } from "bun";

const authCookie = 'pt-auth';

const day = 1000 * 60 * 60 * 24;
let days = Number.parseInt(env.SESSION_DURATION_IN_DAYS ?? '31', 10);
if (Number.isNaN(days)) {
  days = 31;
}

const maxAge = days * day;
const renewalTime = Math.abs(Math.floor(maxAge / 7));

async function login(email: string, password: string): Promise<string | null> {
  const result = await drizzleDB.select().from(users).where(eq(users.email, email.toLocaleLowerCase()));
  if (result.length === 0) {
    return null;
  }

  const user = result[0];
  if (!user) {
    return null;
  }

  // verify password
  const valid = await Bun.password.verify(password, user.hash);
  if (!valid) {
    return null;
  }

  const token = Bun.randomUUIDv7('base64url');

  await drizzleDB.insert(userSessions).values({
    token,
    maxAge: Date.now() + maxAge,
    userId: user.id,
  });

  return token;
}

function loginResponse(token: string) {
  const cookie = new Bun.Cookie({
    name: authCookie,
    value: token,
    path: '/',
    maxAge,
    secure: true,
    sameSite: 'strict',
    httpOnly: true,
  });

  return new Response('Login successful', {
    headers: new Headers({
      'Set-Cookie': cookie.toString(),
    }),
  });
}

function logoutHeaders() {
  return new Headers({
    'Set-Cookie': new Bun.Cookie({
      name: authCookie,
      path: '/',
      maxAge: -1,
      secure: true,
      sameSite: 'strict',
      httpOnly: true,
    }).toString(),
  });
}

function logoutResponse() {
  return new Response('Logout successful', { status: 303, headers: logoutHeaders() });
}

function unAuthorizedResponse() {
  return new Response('Unauthorized', { status: 401, headers: logoutHeaders() });
}

async function verify(headers: Headers) {
  const cookieMap = new Bun.CookieMap(headers.get('cookie') || '');
  const token = cookieMap.get(authCookie);
  if (!token) {
    // cookie not found
    return null;
  }

  const result = await drizzleDB.select().from(userSessions).where(eq(userSessions.token, token));
  if (result.length === 0) {
    // session not found
    return null;
  }

  const session = result[0];
  if (!session) {
    // session not found
    return null;
  }

  const now = Date.now();
  if (session.maxAge <= now) {
    // session expired
    await drizzleDB.delete(userSessions).where(eq(userSessions.token, token));
    return null;
  }

  if (session.maxAge <= now - renewalTime) {
    // renew session
    const newMaxAge = now + maxAge;
    await drizzleDB.update(userSessions).set({ maxAge: newMaxAge }).where(eq(userSessions.token, token));
    session.maxAge = newMaxAge; // renew session
  }

  return session;
}

function verifyResponse(token: string) {
  const cookie = new Bun.Cookie({
    name: authCookie,
    value: token,
    path: '/',
    maxAge,
    secure: true,
    sameSite: 'strict',
    httpOnly: true,
  });

  return new Response('Verify successful', {
    headers: new Headers({ 'Set-Cookie': cookie.toString() }),
  });
}

function verifyFailResponse() {
  const cookie = new Bun.Cookie({
    name: authCookie,
    path: '/',
    maxAge: -1,
    secure: true,
    sameSite: 'strict',
    httpOnly: true,
  });

  return new Response('Verify failure', {
    status: 401,
    headers: new Headers({ 'Set-Cookie': cookie.toString() }),
  });
}

export default {
  login,
  loginResponse,
  logoutResponse,
  unAuthorizedResponse,
  verify,
  verifyResponse,
  verifyFailResponse,
}
